package org.springblade.gateway.filter;

import cn.hutool.core.util.StrUtil;
import cn.hutool.core.util.URLUtil;
import cn.hutool.crypto.SecureUtil;
import cn.hutool.json.JSONUtil;
import com.alibaba.fastjson2.JSON;
import lombok.extern.slf4j.Slf4j;
import org.springblade.core.constant.TokenConstant;
import org.springblade.core.jwt.utils.AuthKit;
import org.springblade.core.secure.BladeUser;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.api.ResultCode;
import org.springblade.core.tool.utils.Base62Util;
import org.springblade.core.tool.utils.WebUtil;
import org.springblade.gateway.iam.BladeUserWrapper;
import org.springblade.gateway.iam.IamInterUtil;
import org.springblade.gateway.iam.IamTokenConstant;
import org.springblade.gateway.iam.SignedPrincipal;
import org.springblade.gateway.properties.BladeSecureProperties;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cloud.gateway.filter.GatewayFilterChain;
import org.springframework.cloud.gateway.filter.GlobalFilter;
import org.springframework.core.Ordered;
import org.springframework.core.io.buffer.DataBufferFactory;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.server.reactive.ServerHttpRequest;
import org.springframework.http.server.reactive.ServerHttpResponse;
import org.springframework.stereotype.Component;
import org.springframework.web.server.ServerWebExchange;
import reactor.core.publisher.Mono;

import java.nio.charset.StandardCharsets;
import java.util.Base64;
import java.util.Collection;
import java.util.List;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;
import java.util.stream.Stream;

@Slf4j
@Component
public class AuthTokenFilter implements GlobalFilter, Ordered {

    @Autowired
    private BladeSecureProperties bladeSecureProperties;

    @Autowired
    private StringRedisTemplate stringRedisTemplate;

    final static String YX_CURRENT_USER_ROLE_KEY_PREFIX = "yx:user:role:";
    final static String YX_CURRENT_USER_DEPT_KEY_PREFIX = "yx:user:dept:";

    final static String YX_CURRENT_USER_LOCK_KEY_PREFIX = "yx:user:lock:";

    @Override
    public Mono<Void> filter(ServerWebExchange exchange, GatewayFilterChain chain) {
        ServerHttpRequest request = exchange.getRequest();
        String uri = request.getURI().getPath();
        //模块需要跳过鉴权的URL
        List<String> moduleSkipUrlList = bladeSecureProperties.getSkipUrl().values().stream().flatMap(item -> item.getUriList().stream().map(f -> item.getPath().concat(f))).collect(Collectors.toList());
        //放行跳过鉴权的URI
        List<String> allSkipUrlList = Stream.of(bladeSecureProperties.getDefaultExcludePatterns(), moduleSkipUrlList).flatMap(Collection::stream).collect(Collectors.toList());
        if (bladeSecureProperties.getEnabled()
                && allSkipUrlList.stream().anyMatch(urlPattern -> WebUtil.pathUrlMatches(urlPattern, uri, false))) {
            return chain.filter(exchange);
        }

        BladeUser bladeUser = null;
        boolean isPrivate = false;
        ServerHttpRequest.Builder mutateBuilder = exchange.getRequest().mutate();
        if (bladeSecureProperties.getType() == 0) { //公司标准产品网关
            //判断Token有效性
            String token = request.getHeaders().getFirst(TokenConstant.HEADER);
            log.debug("token => {}", token);
            bladeUser = AuthKit.getUser(token);
            if (bladeUser == null) {
                return tokenCheckFail(exchange);
            }
            //Token原始字符串传递到服务模块
            mutateBuilder.header(TokenConstant.HEADER, token);
        } else { //研祥网关
            //接口访问IAM网关，然后透传流量带着该header头过来
            String yanxiangUserJson = request.getHeaders().getFirst(IamTokenConstant.HEADER),
                    //接口直接访问业务网关传递token
                    privateUserJsonDec = request.getHeaders().getFirst(bladeSecureProperties.getPlatomixTokenHeader());
            System.out.println("yanxiangUserJson-----------" + yanxiangUserJson);
            if (StrUtil.isNotEmpty(yanxiangUserJson)) {
                SignedPrincipal signedPrincipal = JSONUtil.toBean(URLUtil.decode(yanxiangUserJson), SignedPrincipal.class);
                bladeUser = signedPrincipal.toCurrentUser(bladeSecureProperties.getType());
                //设置部门ID和用户ID
                _setDeptRoleId(bladeUser, signedPrincipal.getInstId(), signedPrincipal.getUserId());
            } else if (StrUtil.isNotEmpty(privateUserJsonDec)) {
                try {
                    byte[] byts = Base64.getDecoder().decode(bladeSecureProperties.getPlatomixTokenAesKey());
                    BladeUserWrapper bladeUserWrapper = JSONUtil.toBean(SecureUtil.aes(byts).decryptStr(privateUserJsonDec), BladeUserWrapper.class);
                    if (System.currentTimeMillis() > bladeUserWrapper.getExpire()) {
                        log.warn("自定义token已超时过期，expire=" + bladeUserWrapper.getExpire());
                        return tokenCheckFail(exchange);
                    }
                    bladeUser = bladeUserWrapper.getBladeUser();
                    if (bladeUser == null || bladeUser.getUserId() == null || bladeUser.getUserId() <= 0 || StrUtil.isEmpty(bladeUser.getTenantId())) {
                        return tokenCheckFail(exchange);
                    }
                    long instId = Base62Util.decode(bladeUser.getTenantId());
                    //设置部门ID和用户ID
                    _setDeptRoleId(bladeUser, instId, bladeUser.getUserId());
                    isPrivate = true;
                } catch (Exception e) {
                    log.error("解析自定义token失败|token=" + privateUserJsonDec, e);
                    return tokenCheckFail(exchange);
                }
            } else {
                log.error("未获取到IAM的用户json数据");
                return tokenCheckFail(exchange);
            }
            if (bladeUser == null) {
                log.error("研祥-用户json数据转换为BladeUser失败");
                return tokenCheckFail(exchange);
            }
            log.debug("type={},isPrivate={},bladeUser={}", bladeSecureProperties.getType(), isPrivate, JSONUtil.toJsonStr(bladeUser));
        }

        String bladeUserJson = JSONUtil.toJsonStr(bladeUser);
        String base64BladeUserJson = Base64.getEncoder().encodeToString(bladeUserJson.getBytes(StandardCharsets.UTF_8));

        //BlaseUserJson base64加密字符串放进来，传递到服务模块中
        mutateBuilder.header(TokenConstant.PLATOMIX_USER, base64BladeUserJson);
        return chain.filter(exchange.mutate().request(mutateBuilder.build()).build());
    }

    private void _setDeptRoleId(BladeUser bladeUser, long instId, long userId) {
        if (bladeSecureProperties.getType() == 2) { //用户端
            //从缓存中获取当前登录人的部门ID和角色ID列表
            String roleKey = YX_CURRENT_USER_ROLE_KEY_PREFIX + userId,
                    deptKey = YX_CURRENT_USER_DEPT_KEY_PREFIX + userId;
            String roleIds = stringRedisTemplate.opsForValue().get(roleKey),
                    deptIds = stringRedisTemplate.opsForValue().get(deptKey);
            if (roleIds == null || deptIds == null) { //缓存中没有数据
                String lockedKey = YX_CURRENT_USER_LOCK_KEY_PREFIX + userId;
                Boolean locked = stringRedisTemplate.opsForValue().setIfAbsent(lockedKey, "1", 30, TimeUnit.SECONDS);
                if (locked != null && locked.booleanValue()) { //已锁定，查询扔到缓存
                    roleIds = IamInterUtil.getRoleIds(bladeSecureProperties.getIamInst9n1mDomain(), instId, userId);
                    deptIds = IamInterUtil.getDeptIds(bladeSecureProperties.getIamInst9n1mDomain(), instId, userId);
                    log.info("从接口查询|userId={},roleIds={},deptIds={}", userId, roleIds, deptIds);
                    if (roleIds != null && deptIds != null) {
                        stringRedisTemplate.opsForValue().set(roleKey, roleIds, bladeSecureProperties.getCurrUserDeptRoleCacheSecs(), TimeUnit.SECONDS);
                        stringRedisTemplate.opsForValue().set(deptKey, deptIds, bladeSecureProperties.getCurrUserDeptRoleCacheSecs(), TimeUnit.SECONDS);
                    }
                }
            }
            if (StrUtil.isNotBlank(roleIds) && StrUtil.isNotBlank(deptIds)) {
                bladeUser.setRoleId(!"0".equals(roleIds) ? roleIds : null);
                bladeUser.setDeptId(!"0".equals(deptIds) ? deptIds : null);
            }
        }
    }

    private Mono<Void> tokenCheckFail(ServerWebExchange exchange) {
        ServerHttpResponse response = exchange.getResponse();
        response.getHeaders().setContentType(MediaType.APPLICATION_JSON);
        response.setStatusCode(HttpStatus.UNAUTHORIZED);

        ResultCode resultCode = ResultCode.UN_AUTHORIZED;
        log.error("[鉴权异常处理]请求路径:{},Code:{},Message:{}", exchange.getRequest().getPath(), resultCode.getCode(), resultCode.getMessage());
        return response.writeWith(Mono.fromSupplier(() -> {
            DataBufferFactory bufferFactory = response.bufferFactory();
            return bufferFactory.wrap(JSON.toJSONBytes(R.fail(resultCode)));
        }));
    }

    @Override
    public int getOrder() {
        return -100;
    }

}
