package org.springblade.gateway.handler;

import com.alibaba.fastjson2.JSON;
import lombok.extern.slf4j.Slf4j;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.WebUtil;
import org.springblade.gateway.properties.BladeSecureProperties;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.web.reactive.error.ErrorWebExceptionHandler;
import org.springframework.cloud.gateway.support.NotFoundException;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.annotation.Order;
import org.springframework.core.io.buffer.DataBufferFactory;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.server.RequestPath;
import org.springframework.http.server.reactive.ServerHttpRequest;
import org.springframework.http.server.reactive.ServerHttpResponse;
import org.springframework.web.server.ResponseStatusException;
import org.springframework.web.server.ServerWebExchange;

import reactor.core.publisher.Mono;

/**
 * 网关统一异常处理
 *
 * @author wookvn
 */
@Order(-1)
@Slf4j
@Configuration
public class GlobalExceptionHandler implements ErrorWebExceptionHandler {

	@Autowired
	private BladeSecureProperties bladeSecureProperties;

	@Override
	public Mono<Void> handle(ServerWebExchange exchange, Throwable ex) {
		ServerHttpResponse response = exchange.getResponse();
		ServerHttpRequest request = exchange.getRequest();

		//白名单放行URL
		RequestPath path = request.getPath();
		if(bladeSecureProperties.getDefaultWhiteUrlPatterns().stream().anyMatch(urlPattern -> WebUtil.pathUrlMatches(urlPattern, path.toString(), false))) {
			return Mono.empty();
		}

		if (response.isCommitted()) {
			return Mono.error(ex);
		}

		String msg;

		if (ex instanceof NotFoundException) {
			msg = "服务未找到";
		} else if (ex instanceof ResponseStatusException) {
			ResponseStatusException responseStatusException = (ResponseStatusException) ex;
			msg = responseStatusException.getMessage();
		} else {
			msg = "内部服务错误";
		}

		log.error("[网关异常处理]请求路径:{},异常信息:{}", path, ex.getMessage());

		response.getHeaders().setContentType(MediaType.APPLICATION_JSON);
		response.setStatusCode(HttpStatus.OK);

		return response.writeWith(Mono.fromSupplier(() -> {
			DataBufferFactory bufferFactory = response.bufferFactory();
			return bufferFactory.wrap(JSON.toJSONBytes(R.fail(msg)));
		}));
	}
}